local md = require "luci.model.modem"
local modem_list = md.get_all_modems()

m = Map("azure_iothub")
s = m:section(NamedSection, "azure_iothub", "azure_iothub", translate("Azure IoT Hub"))
s.addremove = false

o = s:option(Flag, "enable", translate("Enable Azure IoT Hub monitoring"), translate(""))
o.rmempty = false

o = s:option(Value, "connection_string", translate("Connection string"))
o.rmempty = false
o.datatype = "string"
o.maxlength = 150

o = s:option(ListValue, "msg_type", translate("Messages Type"))
if #modem_list > 0 then
	o:value("gsmctl", translate("GSM values"))
end
o:value("mqtt", translate("MQTT messages"))

local msg_type = m:formvalue("cbid.azure_iothub.azure_iothub.msg_type")

o = s:option(Value, "mqtt_ip", translate("MQTT Host"), translate("Host address of MQTT broker"))
if msg_type and msg_type == "mqtt" then
	o.rmempty = false
end
o:depends("msg_type", "mqtt")
o.datatype = "host"

o = s:option(Value, "mqtt_port", translate("MQTT Port"), translate("Port of MQTT broker"))
if msg_type and msg_type == "mqtt" then
	o.rmempty = false
end
o.datatype = "port"
o:depends("msg_type", "mqtt")

o = s:option(Value, "mqtt_topic", translate("Topic"), translate("Topic to subscribe"))
o.datatype = "lengthvalidation(0, 512)"
if msg_type and msg_type == "mqtt" then
	o.rmempty = false
end
o.datatype = "string"
o.maxlength = "64"
o:depends("msg_type", "mqtt")

o = s:option(Value, "mqtt_username", translate("Username"), translate(""))
o:depends("msg_type", "mqtt")

o = s:option(Value, "mqtt_password", translate("Password"), translate(""))
o.datatype = "credentials_validate"
o.noautocomplete = true
o.password = true
o:depends("msg_type", "mqtt")

o = s:option(Value, "message_interval", translate("Message sending interval (sec.)"), translate("Number of seconds to send device data to Azure IoT Hub (min. 10s)"))
o.default = "300"
if msg_type and msg_type == "gsmctl" then
	o.rmempty = false
end
o.datatype = "range(10,99999)"
o:depends("msg_type", "gsmctl")

if #modem_list > 0 then
	o = s:option(ListValue, "interface", translate("Mobile Interface"), translate("Interface name for which IP address and sent/received bytes would be selected"))
	o:depends("msg_type", "gsmctl")

	local modems = false

	m.uci:foreach("network", "interface", function(s)
		if s.modem then
			o:value(s[".name"])
			modems = true
		end
	end)

	if not modems then
		o:value("", translate("No Modem Connected"))
	end
end

function o.write(self, section, value)
	local netm = require "luci.model.network".init()

	m.uci:set(self.config, section, "interface", value)
	local modem = m.uci:get("network", value, "modem")
	m.uci:set(self.config, section, "modem", modem)
	local net = netm:get_network(value)
	local device = net and net:get_interface()
	local ifname = device and device:name() or ""
	m.uci:set(self.config, section, "ifname", ifname)
end

o = s:option(Value, "gsm", translate("GSM Values"))
o.template = "azure_iothub/azure_iothub"
o.widget = "checkbox"
o.cast = "string"
o:depends("msg_type", "gsmctl")

function o.write(self, section, value) end

function m.on_commit(self)
	if msg_type and msg_type == "gsmctl" then
		local selected = m:formvalue("cbid.azure_iothub.azure_iothub.gsm")
		local gsm_values = {ipaddr = "0", bsent = "0", brecv = "0", connstate = "0", netstate = "0", imei = "0", iccid = "0", model = "0", manuf = "0", serial = "0", revision = "0",
			imsi = "0", simstate = "0", pinstate = "0", signal = "0", rscp = "0", ecio = "0", rsrp = "0", sinr = "0", rsrq = "0", cellid = "0", operator = "0", opernum = "0",
			conntype = "0", temp = "0", pincount = "0" }

		if type(selected) == "table" then
			for _, v in ipairs(selected) do
				gsm_values[v] = "1"
			end
		elseif selected then
			gsm_values[selected] = "1"
		end

		for k, v in pairs(gsm_values) do
			m.uci:set("azure_iothub", "azure_iothub", k, v)
		end
	end
end

return m
